@@ -26,6 +26,7 @@ from calibre.utils.localization import canonicalize_lang
 _xpath_cache = {}
 _re_cache = {}
 
+
 def uniq(vals):
     ''' Remove all duplicates from vals, while preserving order.  '''
     vals = vals or ()
@@ -33,9 +34,11 @@ def uniq(vals):
     seen_add = seen.add
     return list(x for x in vals if x not in seen and not seen_add(x))
 
+
 def dump_dict(cats):
     return json.dumps(object_to_unicode(cats or {}), ensure_ascii=False, skipkeys=True)
 
+
 def XPath(x):
     try:
         return _xpath_cache[x]
@@ -43,6 +46,7 @@ def XPath(x):
         _xpath_cache[x] = ans = etree.XPath(x, namespaces=OPF2_NSMAP)
         return ans
 
+
 def regex(r, flags=0):
     try:
         return _re_cache[(r, flags)]
@@ -50,15 +54,18 @@ def regex(r, flags=0):
         _re_cache[(r, flags)] = ans = re.compile(r, flags)
         return ans
 
+
 def remove_refines(e, refines):
     for x in refines[e.get('id')]:
         x.getparent().remove(x)
     refines.pop(e.get('id'), None)
 
+
 def remove_element(e, refines):
     remove_refines(e, refines)
     e.getparent().remove(e)
 
+
 def properties_for_id(item_id, refines):
     ans = {}
     if item_id:
@@ -70,6 +77,7 @@ def properties_for_id(item_id, refines):
                     ans[key] = val
     return ans
 
+
 def properties_for_id_with_scheme(item_id, prefixes, refines):
     ans = {}
     if item_id:
@@ -90,6 +98,7 @@ def properties_for_id_with_scheme(item_id, prefixes, refines):
                     ans[key] = (scheme_ns, scheme, val)
     return ans
 
+
 def getroot(elem):
     while True:
         q = elem.getparent()
@@ -97,6 +106,7 @@ def getroot(elem):
             return elem
         elem = q
 
+
 def ensure_id(elem):
     root = getroot(elem)
     eid = elem.get('id')
@@ -105,19 +115,31 @@ def ensure_id(elem):
         elem.set('id', eid)
     return eid
 
+
 def normalize_whitespace(text):
     if not text:
         return text
     return re.sub(r'\s+', ' ', text).strip()
 
+
 def simple_text(f):
     @wraps(f)
     def wrapper(*args, **kw):
         return normalize_whitespace(f(*args, **kw))
     return wrapper
 
-def items_with_property(root, q):
-    return XPath("./opf:manifest/opf:item[@properties and contains(concat(' ', normalize-space(@properties), ' '), ' %s ')]" % q)(root)
+
+def items_with_property(root, q, prefixes=None):
+    if prefixes is None:
+        prefixes = read_prefixes(root)
+    q = expand_prefix(q, known_prefixes).lower()
+    for item in XPath("./opf:manifest/opf:item[@properties]")(root):
+        for prop in (item.get('properties') or '').lower().split():
+            prop = expand_prefix(prop, prefixes)
+            if prop == q:
+                yield item
+                break
+
 # }}}
 
 # Prefixes {{{
@@ -135,19 +157,27 @@ reserved_prefixes = {
 }
 
 CALIBRE_PREFIX = 'https://calibre-ebook.com'
+known_prefixes = reserved_prefixes.copy()
+known_prefixes['calibre'] = CALIBRE_PREFIX
+
 
 def parse_prefixes(x):
     return {m.group(1):m.group(2) for m in re.finditer(r'(\S+): \s*(\S+)', x)}
 
+
 def read_prefixes(root):
     ans = reserved_prefixes.copy()
     ans.update(parse_prefixes(root.get('prefix') or ''))
     return ans
 
+
 def expand_prefix(raw, prefixes):
     return regex(r'(\S+)\s*:\s*(\S+)').sub(lambda m:(prefixes.get(m.group(1), m.group(1)) + ':' + m.group(2)), raw or '')
 
+
 def ensure_prefix(root, prefixes, prefix, value=None):
+    if prefixes is None:
+        prefixes = read_prefixes(root)
     prefixes[prefix] = value or reserved_prefixes[prefix]
     prefixes = {k:v for k, v in prefixes.iteritems() if reserved_prefixes.get(k) != v}
     if prefixes:
@@ -158,6 +188,8 @@ def ensure_prefix(root, prefixes, prefix, value=None):
 # }}}
 
 # Refines {{{
+
+
 def read_refines(root):
     ans = defaultdict(list)
     for meta in XPath('./opf:metadata/opf:meta[@refines]')(root):
@@ -166,9 +198,11 @@ def read_refines(root):
             ans[r[1:]].append(meta)
     return ans
 
+
 def refdef(prop, val, scheme=None):
     return (prop, val, scheme)
 
+
 def set_refines(elem, existing_refines, *new_refines):
     eid = ensure_id(elem)
     remove_refines(elem, existing_refines)
@@ -184,6 +218,8 @@ def set_refines(elem, existing_refines, *new_refines):
 # }}}
 
 # Identifiers {{{
+
+
 def parse_identifier(ident, val, refines):
     idid = ident.get('id')
     refines = refines[idid]
@@ -224,6 +260,7 @@ def parse_identifier(ident, val, refines):
     prefix, rest = val.partition(':')[::2]
     return finalize(prefix, rest)
 
+
 def read_identifiers(root, prefixes, refines):
     ans = defaultdict(list)
     for ident in XPath('./opf:metadata/dc:identifier')(root):
@@ -234,6 +271,7 @@ def read_identifiers(root, prefixes, refines):
                 ans[scheme].append(val)
     return ans
 
+
 def set_identifiers(root, prefixes, refines, new_identifiers, force_identifiers=False):
     uid = root.get('unique-identifier')
     package_identifier = None
@@ -259,6 +297,7 @@ def set_identifiers(root, prefixes, refines, new_identifiers, force_identifiers=
             p = package_identifier.getparent()
             p.insert(p.index(package_identifier), ident)
 
+
 def identifier_writer(name):
     def writer(root, prefixes, refines, ival=None):
         uid = root.get('unique-identifier')
@@ -287,6 +326,7 @@ set_uuid = identifier_writer('uuid')
 
 # Title {{{
 
+
 def find_main_title(root, refines, remove_blanks=False):
     first_title = main_title = None
     for title in XPath('./opf:metadata/dc:title')(root):
@@ -304,11 +344,13 @@ def find_main_title(root, refines, remove_blanks=False):
         main_title = first_title
     return main_title
 
+
 @simple_text
 def read_title(root, prefixes, refines):
     main_title = find_main_title(root, refines)
     return None if main_title is None else main_title.text.strip()
 
+
 @simple_text
 def read_title_sort(root, prefixes, refines):
     main_title = find_main_title(root, refines)
@@ -322,6 +364,7 @@ def read_title_sort(root, prefixes, refines):
         if ans:
             return ans
 
+
 def set_title(root, prefixes, refines, title, title_sort=None):
     main_title = find_main_title(root, refines, remove_blanks=True)
     if main_title is None:
@@ -337,6 +380,8 @@ def set_title(root, prefixes, refines, title, title_sort=None):
 # }}}
 
 # Languages {{{
+
+
 def read_languages(root, prefixes, refines):
     ans = []
     for lang in XPath('./opf:metadata/dc:language')(root):
@@ -345,6 +390,7 @@ def read_languages(root, prefixes, refines):
             ans.append(val)
     return uniq(ans)
 
+
 def set_languages(root, prefixes, refines, languages):
     opf_languages = []
     for lang in XPath('./opf:metadata/dc:language')(root):
@@ -367,6 +413,7 @@ def set_languages(root, prefixes, refines, languages):
 
 Author = namedtuple('Author', 'name sort')
 
+
 def is_relators_role(props, q):
     role = props.get('role')
     if role:
@@ -374,6 +421,7 @@ def is_relators_role(props, q):
         return role.lower() == q and (scheme_ns is None or (scheme_ns, scheme) == (reserved_prefixes['marc'], 'relators'))
     return False
 
+
 def read_authors(root, prefixes, refines):
     roled_authors, unroled_authors = [], []
 
@@ -403,12 +451,13 @@ def read_authors(root, prefixes, refines):
 
     return uniq(roled_authors or unroled_authors)
 
+
 def set_authors(root, prefixes, refines, authors):
     ensure_prefix(root, prefixes, 'marc')
     for item in XPath('./opf:metadata/dc:creator')(root):
         props = properties_for_id_with_scheme(item.get('id'), prefixes, refines)
         opf_role = item.get(OPF('role'))
-        if (is_relators_role(props, 'aut')) or (opf_role and opf_role.lower() != 'aut'):
+        if (opf_role and opf_role.lower() != 'aut') or (props.get('role') and not is_relators_role(props, 'aut')):
             continue
         remove_element(item, refines)
     metadata = XPath('./opf:metadata')(root)[0]
@@ -425,6 +474,7 @@ def set_authors(root, prefixes, refines, authors):
             m.text = author.sort
             metadata.append(m)
 
+
 def read_book_producers(root, prefixes, refines):
     ans = []
     for item in XPath('./opf:metadata/dc:contributor')(root):
@@ -440,11 +490,12 @@ def read_book_producers(root, prefixes, refines):
                 ans.append(normalize_whitespace(val))
     return ans
 
+
 def set_book_producers(root, prefixes, refines, producers):
     for item in XPath('./opf:metadata/dc:contributor')(root):
         props = properties_for_id_with_scheme(item.get('id'), prefixes, refines)
         opf_role = item.get(OPF('role'))
-        if (is_relators_role(props, 'bkp')) or (opf_role and opf_role.lower() != 'bkp'):
+        if (opf_role and opf_role.lower() != 'bkp') or (props.get('role') and not is_relators_role(props, 'bkp')):
             continue
         remove_element(item, refines)
     metadata = XPath('./opf:metadata')(root)[0]
@@ -460,6 +511,7 @@ def set_book_producers(root, prefixes, refines, producers):
 
 # Dates {{{
 
+
 def parse_date(raw, is_w3cdtf=False):
     raw = raw.strip()
     if is_w3cdtf:
@@ -472,6 +524,7 @@ def parse_date(raw, is_w3cdtf=False):
             ans = fix_only_date(ans)
     return ans
 
+
 def read_pubdate(root, prefixes, refines):
     for date in XPath('./opf:metadata/dc:date')(root):
         val = (date.text or '').strip()
@@ -481,6 +534,7 @@ def read_pubdate(root, prefixes, refines):
             except Exception:
                 continue
 
+
 def set_pubdate(root, prefixes, refines, val):
     for date in XPath('./opf:metadata/dc:date')(root):
         remove_element(date, refines)
@@ -491,6 +545,7 @@ def set_pubdate(root, prefixes, refines, val):
         d.text = val
         m.append(d)
 
+
 def read_timestamp(root, prefixes, refines):
     pq = '%s:timestamp' % CALIBRE_PREFIX
     sq = '%s:w3cdtf' % reserved_prefixes['dcterms']
@@ -512,6 +567,7 @@ def read_timestamp(root, prefixes, refines):
             except Exception:
                 continue
 
+
 def set_timestamp(root, prefixes, refines, val):
     ensure_prefix(root, prefixes, 'calibre', CALIBRE_PREFIX)
     ensure_prefix(root, prefixes, 'dcterms')
@@ -545,6 +601,7 @@ def read_last_modified(root, prefixes, refines):
 
 # Comments {{{
 
+
 def read_comments(root, prefixes, refines):
     ans = ''
     for dc in XPath('./opf:metadata/dc:description')(root):
@@ -552,6 +609,7 @@ def read_comments(root, prefixes, refines):
             ans += '\n' + dc.text.strip()
     return ans.strip()
 
+
 def set_comments(root, prefixes, refines, val):
     for dc in XPath('./opf:metadata/dc:description')(root):
         remove_element(dc, refines)
@@ -566,12 +624,14 @@ def set_comments(root, prefixes, refines, val):
 
 # Publisher {{{
 
+
 @simple_text
 def read_publisher(root, prefixes, refines):
     for dc in XPath('./opf:metadata/dc:publisher')(root):
         if dc.text:
             return dc.text
 
+
 def set_publisher(root, prefixes, refines, val):
     for dc in XPath('./opf:metadata/dc:publisher')(root):
         remove_element(dc, refines)
@@ -586,6 +646,7 @@ def set_publisher(root, prefixes, refines, val):
 
 # Tags {{{
 
+
 def read_tags(root, prefixes, refines):
     ans = []
     for dc in XPath('./opf:metadata/dc:subject')(root):
@@ -593,6 +654,7 @@ def read_tags(root, prefixes, refines):
             ans.extend(map(normalize_whitespace, dc.text.split(',')))
     return uniq(filter(None, ans))
 
+
 def set_tags(root, prefixes, refines, val):
     for dc in XPath('./opf:metadata/dc:subject')(root):
         remove_element(dc, refines)
@@ -608,6 +670,7 @@ def set_tags(root, prefixes, refines, val):
 
 # Rating {{{
 
+
 def read_rating(root, prefixes, refines):
     pq = '%s:rating' % CALIBRE_PREFIX
     for meta in XPath('./opf:metadata/opf:meta[@property]')(root):
@@ -627,6 +690,7 @@ def read_rating(root, prefixes, refines):
             except Exception:
                 continue
 
+
 def set_rating(root, prefixes, refines, val):
     pq = '%s:rating' % CALIBRE_PREFIX
     for meta in XPath('./opf:metadata/opf:meta[@name="calibre:rating"]')(root):
@@ -645,6 +709,7 @@ def set_rating(root, prefixes, refines, val):
 
 # Series {{{
 
+
 def read_series(root, prefixes, refines):
     series_index = 1.0
     for meta in XPath('./opf:metadata/opf:meta[@property="belongs-to-collection" and @id]')(root):
@@ -669,6 +734,7 @@ def read_series(root, prefixes, refines):
             return s, series_index
     return None, series_index
 
+
 def set_series(root, prefixes, refines, series, series_index):
     for meta in XPath('./opf:metadata/opf:meta[@name="calibre:series" or @name="calibre:series_index"]')(root):
         remove_element(meta, refines)
@@ -684,6 +750,7 @@ def set_series(root, prefixes, refines, series, series_index):
 
 # User metadata {{{
 
+
 def dict_reader(name, load=json.loads, try2=True):
     pq = '%s:%s' % (CALIBRE_PREFIX, name)
 
@@ -714,6 +781,7 @@ def dict_reader(name, load=json.loads, try2=True):
 read_user_categories = dict_reader('user_categories')
 read_author_link_map = dict_reader('author_link_map')
 
+
 def dict_writer(name, serialize=dump_dict, remove2=True):
     pq = '%s:%s' % (CALIBRE_PREFIX, name)
 
@@ -736,6 +804,7 @@ def dict_writer(name, serialize=dump_dict, remove2=True):
 set_user_categories = dict_writer('user_categories')
 set_author_link_map = dict_writer('author_link_map')
 
+
 def deserialize_user_metadata(val):
     val = json.loads(val, object_hook=from_json)
     ans = {}
@@ -745,6 +814,7 @@ def deserialize_user_metadata(val):
     return ans
 read_user_metadata3 = dict_reader('user_metadata', load=deserialize_user_metadata, try2=False)
 
+
 def read_user_metadata2(root):
     ans = {}
     for meta in XPath('./opf:metadata/opf:meta[starts-with(@name, "calibre:user_metadata:")]')(root):
@@ -764,14 +834,17 @@ def read_user_metadata2(root):
             continue
     return ans
 
+
 def read_user_metadata(root, prefixes, refines):
     return read_user_metadata3(root, prefixes, refines) or read_user_metadata2(root)
 
+
 def serialize_user_metadata(val):
     return json.dumps(object_to_unicode(val), ensure_ascii=False, default=to_json, indent=2, sort_keys=True)
 
 set_user_metadata3 = dict_writer('user_metadata', serialize=serialize_user_metadata, remove2=False)
 
+
 def set_user_metadata(root, prefixes, refines, val):
     for meta in XPath('./opf:metadata/opf:meta[starts-with(@name, "calibre:user_metadata:")]')(root):
         remove_element(meta, refines)
@@ -787,6 +860,7 @@ def set_user_metadata(root, prefixes, refines, val):
 
 # Covers {{{
 
+
 def read_raster_cover(root, prefixes, refines):
 
     def get_href(item):
@@ -796,7 +870,7 @@ def read_raster_cover(root, prefixes, refines):
             if href:
                 return href
 
-    for item in items_with_property(root, 'cover-image'):
+    for item in items_with_property(root, 'cover-image', prefixes):
         href = get_href(item)
         if href:
             return href
@@ -808,10 +882,11 @@ def read_raster_cover(root, prefixes, refines):
                 if href:
                     return href
 
+
 def ensure_is_only_raster_cover(root, prefixes, refines, raster_cover_item_href):
     for item in XPath('./opf:metadata/opf:meta[@name="cover"]')(root):
         remove_element(item, refines)
-    for item in items_with_property(root, 'cover-image'):
+    for item in items_with_property(root, 'cover-image', prefixes):
         prop = normalize_whitespace(item.get('properties').replace('cover-image', ''))
         if prop:
             item.set('properties', prop)
@@ -825,12 +900,14 @@ def ensure_is_only_raster_cover(root, prefixes, refines, raster_cover_item_href)
 
 # Reading/setting Metadata objects {{{
 
+
 def first_spine_item(root, prefixes, refines):
     for i in XPath('./opf:spine/opf:itemref/@idref')(root):
         for item in XPath('./opf:manifest/opf:item')(root):
             if item.get('id') == i:
                 return item.get('href') or None
 
+
 def read_metadata(root, ver=None, return_extra_data=False):
     ans = Metadata(_('Unknown'), [_('Unknown')])
     prefixes, refines = read_prefixes(root), read_refines(root)
@@ -879,10 +956,12 @@ def read_metadata(root, ver=None, return_extra_data=False):
         ans = ans, ver, read_raster_cover(root, prefixes, refines), first_spine_item(root, prefixes, refines)
     return ans
 
+
 def get_metadata(stream):
     root = parse_opf(stream)
     return read_metadata(root)
 
+
 def apply_metadata(root, mi, cover_prefix='', cover_data=None, apply_null=False, update_timestamp=False, force_identifiers=False, add_missing_cover=True):
     prefixes, refines = read_prefixes(root), read_refines(root)
     current_mi = read_metadata(root)
@@ -966,6 +1045,7 @@ def apply_metadata(root, mi, cover_prefix='', cover_data=None, apply_null=False,
     pretty_print_opf(root)
     return raster_cover
 
+
 def set_metadata(stream, mi, cover_prefix='', cover_data=None, apply_null=False, update_timestamp=False, force_identifiers=False, add_missing_cover=True):
     root = parse_opf(stream)
     return apply_metadata(
