@@ -24,6 +24,7 @@ from calibre.ebooks.chardet import xml_to_unicode
 from calibre.ebooks.conversion.plugins.epub_input import (
     ADOBE_OBFUSCATION, IDPF_OBFUSCATION, decrypt_font_data)
 from calibre.ebooks.conversion.preprocess import HTMLPreProcessor, CSSPreProcessor as cssp
+from calibre.ebooks.metadata.opf3 import read_prefixes, items_with_property, ensure_prefix, CALIBRE_PREFIX
 from calibre.ebooks.metadata.utils import parse_opf_version
 from calibre.ebooks.mobi import MobiError
 from calibre.ebooks.mobi.reader.headers import MetadataHeader
@@ -47,11 +48,13 @@ exists, join, relpath = os.path.exists, os.path.join, os.path.relpath
 OEB_FONTS = {guess_type('a.ttf'), guess_type('b.otf'), guess_type('a.woff'), 'application/x-font-ttf', 'application/x-font-otf', 'application/font-sfnt'}
 OPF_NAMESPACES = {'opf':OPF2_NS, 'dc':DC11_NS}
 
+
 class CSSPreProcessor(cssp):
 
     def __call__(self, data):
         return self.MS_PAT.sub(self.ms_sub, data)
 
+
 def clone_dir(src, dest):
     ' Clone a directory using hard links for the files, dest must already exist '
     for x in os.listdir(src):
@@ -66,6 +69,7 @@ def clone_dir(src, dest):
             except:
                 shutil.copy2(spath, dpath)
 
+
 def clone_container(container, dest_dir):
     ' Efficiently clone a container using hard links '
     dest_dir = os.path.abspath(os.path.realpath(dest_dir))
@@ -75,21 +79,28 @@ def clone_container(container, dest_dir):
         return cls(None, None, container.log, clone_data=clone_data)
     return cls(None, container.log, clone_data=clone_data)
 
+
 def name_to_abspath(name, root):
     return os.path.abspath(join(root, *name.split('/')))
 
+
 def abspath_to_name(path, root):
     return relpath(os.path.abspath(path), root).replace(os.sep, '/')
 
+
 def name_to_href(name, root, base=None, quote=urlquote):
     fullpath = name_to_abspath(name, root)
     basepath = root if base is None else os.path.dirname(name_to_abspath(base, root))
     path = relpath(fullpath, basepath).replace(os.sep, '/')
     return quote(path)
 
+
 def href_to_name(href, root, base=None):
     base = root if base is None else os.path.dirname(name_to_abspath(base, root))
-    purl = urlparse(href)
+    try:
+        purl = urlparse(href)
+    except ValueError:
+        return None
     if purl.scheme or not purl.path:
         return None
     href = urlunquote(purl.path)
@@ -101,6 +112,7 @@ def href_to_name(href, root, base=None):
     fullpath = os.path.join(base, *href.split('/'))
     return abspath_to_name(fullpath, root)
 
+
 class ContainerBase(object):  # {{{
     '''
     A base class that implements just the parsing methods. Useful to create
@@ -189,6 +201,7 @@ class ContainerBase(object):  # {{{
                          css_preprocessor=(None if self.tweak_mode else self.css_preprocessor))
 # }}}
 
+
 class Container(ContainerBase):  # {{{
 
     '''
@@ -334,7 +347,10 @@ class Container(ContainerBase):  # {{{
         if not os.path.exists(base):
             os.makedirs(base)
         with lopen(path, 'wb') as f:
-            f.write(data)
+            if hasattr(data, 'read'):
+                shutil.copyfileobj(data, f)
+            else:
+                f.write(data)
         mt = media_type or self.guess_type(name)
         self.name_path_map[name] = path
         self.mime_map[name] = mt
@@ -612,10 +628,10 @@ class Container(ContainerBase):  # {{{
 
     def manifest_items_with_property(self, property_name):
         ' All manifest items that have the specified property '
-        q = property_name.lower()
-        for item in self.opf_xpath('//opf:manifest/opf:item[@href and @properties]'):
-            props = (item.get('properties') or '').lower().split()
-            if q in props:
+        prefixes = read_prefixes(self.opf)
+        for item in items_with_property(self.opf, property_name, prefixes):
+            href = item.get('href')
+            if href:
                 yield self.href_to_name(item.get('href'), self.opf_name)
 
     def manifest_items_of_type(self, predicate):
@@ -631,6 +647,38 @@ class Container(ContainerBase):  # {{{
                 for name in names:
                     yield name
 
+    def apply_unique_properties(self, name, *properties):
+        ''' Ensure that the specified properties are set on only the manifest item
+        identified by name. You can pass None as the name to remove the
+        property from all items. '''
+        properties = frozenset(properties)
+        removed_names, added_names = [], []
+        for p in properties:
+            if p.startswith('calibre:'):
+                ensure_prefix(self.opf, None, 'calibre', CALIBRE_PREFIX)
+                break
+
+        for item in self.opf_xpath('//opf:manifest/opf:item'):
+            iname = self.href_to_name(item.get('href'), self.opf_name)
+            props = (item.get('properties') or '').split()
+            lprops = {p.lower() for p in props}
+            for prop in properties:
+                if prop.lower() in lprops:
+                    if name != iname:
+                        removed_names.append(iname)
+                        props = [p for p in props if p.lower() != prop]
+                        if props:
+                            item.set('properties', ' '.join(props))
+                        else:
+                            del item.attrib['properties']
+                else:
+                    if name == iname:
+                        added_names.append(iname)
+                        props.append(prop)
+                        item.set('properties', ' '.join(props))
+        self.dirty(self.opf_name)
+        return removed_names, added_names
+
     @property
     def guide_type_map(self):
         ' Mapping of guide type to canonical name '
@@ -971,9 +1019,12 @@ class Container(ContainerBase):  # {{{
 # }}}
 
 # EPUB {{{
+
+
 class InvalidEpub(InvalidBook):
     pass
 
+
 class ObfuscationKeyMissing(InvalidEpub):
     pass
 
@@ -981,6 +1032,7 @@ OCF_NS = 'urn:oasis:names:tc:opendocument:xmlns:container'
 VCS_IGNORE_FILES = frozenset('.gitignore .hgignore .agignore .bzrignore'.split())
 VCS_DIRS = frozenset(('.git', '.hg', '.svn', '.bzr'))
 
+
 def walk_dir(basedir):
     for dirpath, dirnames, filenames in os.walk(basedir):
         for vcsdir in VCS_DIRS:
@@ -994,6 +1046,7 @@ def walk_dir(basedir):
             if fname not in VCS_IGNORE_FILES:
                 yield is_root, dirpath, fname
 
+
 class EpubContainer(Container):
 
     book_type = 'epub'
@@ -1162,7 +1215,7 @@ class EpubContainer(Container):
                     break
         if raw_unique_identifier is not None:
             idpf_key = raw_unique_identifier
-            idpf_key = re.sub(u'\u0020\u0009\u000d\u000a', u'', idpf_key)
+            idpf_key = re.sub(u'[\u0020\u0009\u000d\u000a]', u'', idpf_key)
             idpf_key = hashlib.sha1(idpf_key.encode('utf-8')).digest()
         key = None
         for item in self.opf_xpath('//*[local-name()="metadata"]/*'
@@ -1244,6 +1297,7 @@ class EpubContainer(Container):
     def path_to_ebook(self):
         def fget(self):
             return self.pathtoepub
+
         def fset(self, val):
             self.pathtoepub = val
         return property(fget=fget, fset=fset)
@@ -1251,9 +1305,12 @@ class EpubContainer(Container):
 # }}}
 
 # AZW3 {{{
+
+
 class InvalidMobi(InvalidBook):
     pass
 
+
 def do_explode(path, dest):
     from calibre.ebooks.mobi.reader.mobi6 import MobiReader
     from calibre.ebooks.mobi.reader.mobi8 import Mobi8Reader
@@ -1290,11 +1347,13 @@ def opf_to_azw3(opf, outpath, container):
     set_cover(oeb)
     outp.convert(oeb, outpath, inp, plumber.opts, container.log)
 
+
 def epub_to_azw3(epub, outpath=None):
     container = get_container(epub, tweak_mode=True)
     outpath = outpath or (epub.rpartition('.')[0] + '.azw3')
     opf_to_azw3(container.name_to_abspath(container.opf_name), outpath, container)
 
+
 class AZW3Container(Container):
 
     book_type = 'azw3'
@@ -1365,6 +1424,7 @@ class AZW3Container(Container):
     def path_to_ebook(self):
         def fget(self):
             return self.pathtoazw3
+
         def fset(self, val):
             self.pathtoazw3 = val
         return property(fget=fget, fset=fset)
@@ -1374,6 +1434,7 @@ class AZW3Container(Container):
         return set(self.name_path_map)
 # }}}
 
+
 def get_container(path, log=None, tdir=None, tweak_mode=False):
     if log is None:
         log = default_log
@@ -1386,6 +1447,7 @@ def get_container(path, log=None, tdir=None, tweak_mode=False):
     ebook.tweak_mode = tweak_mode
     return ebook
 
+
 def test_roundtrip():
     ebook = get_container(sys.argv[-1])
     p = PersistentTemporaryFile(suffix='.'+sys.argv[-1].rpartition('.')[-1])
@@ -1399,5 +1461,3 @@ def test_roundtrip():
 
 if __name__ == '__main__':
     test_roundtrip()
-
-
