@@ -27,6 +27,8 @@ from calibre.utils.img import image_from_x, blend_image
 from calibre.utils.localization import is_rtl
 
 _css = None
+
+
 def css():
     global _css
     if _css is None:
@@ -89,6 +91,7 @@ def render_html(mi, css, vertical, widget, all_fields=False, render_data_func=No
                 % (table, right_pane))
     return ans
 
+
 def get_field_list(fm, use_defaults=False):
     from calibre.gui2.ui import get_gui
     db = get_gui().current_db
@@ -109,6 +112,7 @@ def get_field_list(fm, use_defaults=False):
     available = frozenset(fm.displayable_field_keys())
     return [(f, d) for f, d in fieldlist if f in available]
 
+
 def render_data(mi, use_roman_numbers=True, all_fields=False):
     field_list = get_field_list(getattr(mi, 'field_metadata', field_metadata))
     field_list = [(x, all_fields or display) for x, display in field_list]
@@ -117,6 +121,7 @@ def render_data(mi, use_roman_numbers=True, all_fields=False):
 
 # }}}
 
+
 def details_context_menu_event(view, ev, book_info):  # {{{
     p = view.page()
     mf = p.mainFrame()
@@ -210,6 +215,7 @@ def details_context_menu_event(view, ev, book_info):  # {{{
         menu.exec_(ev.globalPos())
 # }}}
 
+
 class CoverView(QWidget):  # {{{
 
     cover_changed = pyqtSignal(object, object)
@@ -231,7 +237,7 @@ class CoverView(QWidget):  # {{{
                 QSizePolicy.Expanding if vertical else QSizePolicy.Minimum,
                 QSizePolicy.Expanding)
 
-        self.default_pixmap = QPixmap(I('book.png'))
+        self.default_pixmap = QPixmap(I('default_cover.png'))
         self.pixmap = self.default_pixmap
         self.pwidth = self.pheight = None
         self.data = {}
@@ -414,6 +420,7 @@ class CoverView(QWidget):  # {{{
 
 # Book Info {{{
 
+
 class BookInfo(QWebView):
 
     link_clicked = pyqtSignal(object)
@@ -444,7 +451,7 @@ class BookInfo(QWebView):
             ('remove_format', 'trash.png'), ('save_format', 'save.png'),
             ('restore_format', 'edit-undo.png'), ('copy_link','edit-copy.png'),
             ('manage_author', 'user_profile.png'), ('compare_format', 'diff.png'),
-            ('set_cover_format', 'book.png'),
+            ('set_cover_format', 'default_cover.png'),
         ]:
             ac = QAction(QIcon(I(icon)), '', self)
             ac.current_fmt = None
@@ -619,6 +626,7 @@ class DetailsLayout(QLayout):  # {{{
 
 # }}}
 
+
 class BookDetails(QWidget):  # {{{
 
     show_book_info = pyqtSignal()
