@@ -14,6 +14,7 @@ from PyQt5.Qt import (QDialog, QIcon, QApplication, QSize, QKeySequence,
 from calibre.constants import __version__, isfrozen
 from calibre.gui2 import gprefs
 
+
 class MessageBox(QDialog):  # {{{
 
     ERROR = 0
@@ -166,6 +167,7 @@ class MessageBox(QDialog):  # {{{
         self.resize_needed.emit()
 # }}}
 
+
 class ViewLog(QDialog):  # {{{
 
     def __init__(self, title, html, parent=None, unique_name=None):
@@ -203,12 +205,12 @@ class ViewLog(QDialog):  # {{{
         QApplication.clipboard().setText(txt)
 
     def dialog_closing(self, result):
-        self.geom = bytearray(self.saveGeometry())
-        gprefs[self.unique_name] = self.geom
+        gprefs[self.unique_name] = bytearray(self.saveGeometry())
 # }}}
 
 _proceed_memory = []
 
+
 class ProceedNotification(MessageBox):  # {{{
 
     '''
@@ -280,6 +282,7 @@ class ProceedNotification(MessageBox):  # {{{
 
 # }}}
 
+
 class ErrorNotification(MessageBox):  # {{{
 
     def __init__(self, html_log, log_viewer_title, title, msg,
@@ -320,6 +323,7 @@ class ErrorNotification(MessageBox):  # {{{
         _proceed_memory.remove(self)
 # }}}
 
+
 class JobError(QDialog):  # {{{
 
     WIDTH = 600
