@@ -23,14 +23,16 @@ from calibre.customize.ui import (
 from calibre.gui2 import error_dialog, question_dialog, info_dialog, open_url, gprefs
 from calibre.gui2.preferences.plugins import ConfigWidget
 from calibre.utils.date import UNDEFINED_DATE, format_date
+from calibre.utils.https import get_https_resource_securely
 
-SERVER = 'https://plugins.calibre-ebook.com/'
+SERVER = 'https://code.calibre-ebook.com/plugins/'
 INDEX_URL = '%splugins.json.bz2' % SERVER
 FILTER_ALL = 0
 FILTER_INSTALLED = 1
 FILTER_UPDATE_AVAILABLE = 2
 FILTER_NOT_INSTALLED = 3
 
+
 def get_plugin_updates_available(raise_error=False):
     '''
     API exposed to read whether there are updates available for any
@@ -47,18 +49,20 @@ def get_plugin_updates_available(raise_error=False):
             return update_plugins
     return None
 
+
 def filter_upgradeable_plugins(display_plugin):
     return display_plugin.is_upgrade_available()
 
+
 def filter_not_installed_plugins(display_plugin):
     return not display_plugin.is_installed()
 
+
 def read_available_plugins(raise_error=False):
     import json, bz2
     display_plugins = []
-    br = browser()
     try:
-        raw = br.open_novisit(INDEX_URL).read()
+        raw = get_https_resource_securely(INDEX_URL)
         if not raw:
             return
         raw = json.loads(bz2.decompress(raw))
@@ -81,6 +85,7 @@ def read_available_plugins(raise_error=False):
     display_plugins = sorted(display_plugins, key=lambda k: k.name)
     return display_plugins
 
+
 def get_installed_plugin_status(display_plugin):
     display_plugin.installed_version = None
     display_plugin.plugin = None
@@ -110,6 +115,7 @@ class ImageTitleLayout(QHBoxLayout):
     '''
     A reusable layout widget displaying an image followed by a title
     '''
+
     def __init__(self, parent, icon_name, title):
         QHBoxLayout.__init__(self)
         title_font = QFont()
@@ -430,7 +436,14 @@ class PluginUpdaterDialog(SizePersistedDialog):
         self._initialize_controls()
         self._create_context_menu()
 
-        display_plugins = read_available_plugins()
+        try:
+            display_plugins = read_available_plugins(raise_error=True)
+        except Exception:
+            display_plugins = []
+            import traceback
+            error_dialog(self.gui, _('Update Check Failed'),
+                        _('Unable to reach the plugin index page.'),
+                        det_msg=traceback.format_exc(), show=True)
 
         if display_plugins:
             self.model = DisplayPluginModel(display_plugins)
@@ -443,9 +456,6 @@ class PluginUpdaterDialog(SizePersistedDialog):
             self.filter_combo.setCurrentIndex(initial_filter)
             self._select_and_focus_view()
         else:
-            error_dialog(self.gui, _('Update Check Failed'),
-                        _('Unable to reach the plugin index page.'),
-                        det_msg=INDEX_URL, show=True)
             self.filter_combo.setEnabled(False)
         # Cause our dialog size to be restored from prefs or created on first usage
         self.resize_dialog()
@@ -725,6 +735,7 @@ class PluginUpdaterDialog(SizePersistedDialog):
             b = d.bb.addButton(_('Restart calibre now'), d.bb.AcceptRole)
             b.setIcon(QIcon(I('lt.png')))
             d.do_restart = False
+
             def rf():
                 d.do_restart = True
             b.clicked.connect(rf)
@@ -836,8 +847,7 @@ class PluginUpdaterDialog(SizePersistedDialog):
 
     def _download_zip(self, plugin_zip_url):
         from calibre.ptempfile import PersistentTemporaryFile
-        br = browser(user_agent='%s %s' % (__appname__, __version__))
-        raw = br.open_novisit(plugin_zip_url).read()
+        raw = get_https_resource_securely(plugin_zip_url, headers={'User-Agent':'%s %s' % (__appname__, __version__)})
         with PersistentTemporaryFile('.zip') as pt:
             pt.write(raw)
         return pt.name
