@@ -47,12 +47,14 @@ BASE_URL = 'https://code.calibre-ebook.com/icon-themes/'
 
 COVER_SIZE = (340, 272)
 
+
 def render_svg(filepath):
     must_use_qt(headless=False)
     pngpath = filepath[:-4] + '.png'
     i = QImage(filepath)
     i.save(pngpath)
 
+
 def read_images_from_folder(path):
     name_map = {}
     path = os.path.abspath(path)
@@ -71,12 +73,14 @@ def read_images_from_folder(path):
             name_map[name] = filepath
     return name_map
 
+
 class Theme(object):
 
     def __init__(self, title='', author='', version=-1, description='', license='Unknown', url=None, cover=None):
         self.title, self.author, self.version, self.description = title, author, version, description
         self.license, self.cover, self.url = license, cover, url
 
+
 class Report(object):
 
     def __init__(self, path, name_map, extra, missing, theme):
@@ -87,6 +91,7 @@ class Report(object):
     def name(self):
         return ascii_filename(self.theme.title).replace(' ', '_').replace('.', '_').lower()
 
+
 def read_theme_from_folder(path):
     path = os.path.abspath(path)
     current_image_map = read_images_from_folder(P('images', allow_user_override=False))
@@ -107,6 +112,7 @@ def read_theme_from_folder(path):
     except ValueError:
         # Corrupted metadata file
         metadata = {}
+
     def safe_int(x):
         try:
             return int(x)
@@ -125,6 +131,7 @@ def read_theme_from_folder(path):
         theme.cover = create_cover(ans)
     return ans
 
+
 def icon_for_action(name):
     for plugin in interface_actions():
         if plugin.name == name:
@@ -135,6 +142,7 @@ def icon_for_action(name):
             if icon:
                 return icon
 
+
 def default_cover_icons(cols=5):
     count = 0
     for ac in gprefs.defaults['action-layout-toolbar']:
@@ -152,6 +160,7 @@ def default_cover_icons(cols=5):
         del extra[0]
         count += 1
 
+
 def create_cover(report, icons=(), cols=5, size=120, padding=16):
     icons = icons or tuple(default_cover_icons(cols))
     rows = int(math.ceil(len(icons) / cols))
@@ -176,6 +185,7 @@ def create_cover(report, icons=(), cols=5, size=120, padding=16):
             canvas.compose(img, x + dx, y)
     return canvas.export()
 
+
 def verify_theme(report):
     must_use_qt()
     report.bad = bad = {}
@@ -186,6 +196,7 @@ def verify_theme(report):
             bad[name] = reader.errorString()
     return bool(bad)
 
+
 class ThemeCreateDialog(Dialog):
 
     def __init__(self, parent, report):
@@ -284,6 +295,7 @@ class ThemeCreateDialog(Dialog):
                 'You must specify an author for this icon theme'), show=True)
         return Dialog.accept(self)
 
+
 class Compress(QProgressDialog):
 
     update_signal = pyqtSignal(object, object)
@@ -325,6 +337,7 @@ class Compress(QProgressDialog):
         else:
             self.update_signal.emit(self.maximum(), '')
 
+
 def create_themeball(report, progress=None, abort=None):
     pool = ThreadPool(processes=cpu_count())
     buf = BytesIO()
@@ -412,6 +425,7 @@ def create_theme(folder=None, parent=None):
 
 # Choose Theme  {{{
 
+
 def download_cover(cover_url, etag=None, cached=b''):
     url = BASE_URL + cover_url
     headers = {}
@@ -429,6 +443,7 @@ def download_cover(cover_url, etag=None, cached=b''):
             return cached, etag
         raise
 
+
 def get_cover(metadata):
     cdir = os.path.join(cache_dir(), 'icon-theme-covers')
     try:
@@ -436,9 +451,11 @@ def get_cover(metadata):
     except EnvironmentError as e:
         if e.errno != errno.EEXIST:
             raise
+
     def path(ext):
         return os.path.join(cdir, metadata['name'] + '.' + ext)
     etag_file, cover_file = map(path, 'etag jpg'.split())
+
     def safe_read(path):
         try:
             with open(path, 'rb') as f:
@@ -457,6 +474,7 @@ def get_cover(metadata):
             f.write(etag)
     return cached or b''
 
+
 def get_covers(themes, callback, num_of_workers=8):
     items = Queue()
     tuple(map(items.put, themes))
@@ -481,6 +499,7 @@ def get_covers(themes, callback, num_of_workers=8):
         t.daemon = True
         t.start()
 
+
 class Delegate(QStyledItemDelegate):
 
     SPACING = 10
@@ -517,6 +536,7 @@ class Delegate(QStyledItemDelegate):
         painter.drawStaticText(COVER_SIZE[0] + self.SPACING, option.rect.top() + self.SPACING, theme['static-text'])
         painter.restore()
 
+
 class DownloadProgress(ProgressDialog):
 
     ds = pyqtSignal(object)
@@ -539,6 +559,7 @@ class DownloadProgress(ProgressDialog):
     def queue_reject(self):
         self.rej.emit()
 
+
 class ChooseTheme(Dialog):
 
     cover_downloaded = pyqtSignal(object, object)
@@ -581,6 +602,7 @@ class ChooseTheme(Dialog):
         self.w = w = QWidget(self)
         l.addWidget(w)
         w.l = l = QGridLayout(w)
+
         def add_row(x, y=None):
             if isinstance(x, type('')):
                 x = QLabel(x)
@@ -688,9 +710,13 @@ class ChooseTheme(Dialog):
 
     def set_cover(self, theme, cdata):
         theme['cover-pixmap'] = p = QPixmap()
+        try:
+            dpr = self.devicePixelRatioF()
+        except AttributeError:
+            dpr = self.devicePixelRatio()
         if isinstance(cdata, bytes):
             p.loadFromData(cdata)
-            p.setDevicePixelRatio(self.devicePixelRatio())
+            p.setDevicePixelRatio(dpr)
         item = self.item_from_name(theme['name'])
         if item is not None:
             item.setData(Qt.DecorationRole, p)
@@ -745,6 +771,7 @@ class ChooseTheme(Dialog):
         if ret == d.Rejected or not self.keep_downloading or d.canceled or self.downloaded_theme is None:
             return
         dt = self.downloaded_theme
+
         def commit_changes():
             dt.seek(0)
             f = decompress(dt)
@@ -757,6 +784,7 @@ class ChooseTheme(Dialog):
 
 # }}}
 
+
 def remove_icon_theme():
     icdir = os.path.join(config_dir, 'resources', 'images')
     metadata_file = os.path.join(icdir, 'icon-theme.json')
@@ -775,12 +803,14 @@ def remove_icon_theme():
                 raise
     os.remove(metadata_file)
 
+
 def safe_copy(src, destpath):
     tpath = destpath + '-temp'
     with open(tpath, 'wb') as dest:
         shutil.copyfileobj(src, dest)
     atomic_rename(tpath, destpath)
 
+
 def install_icon_theme(theme, f):
     icdir = os.path.abspath(os.path.join(config_dir, 'resources', 'images'))
     if not os.path.exists(icdir):
