@@ -15,12 +15,13 @@ from lxml.builder import ElementMaker
 
 from calibre import force_unicode
 from calibre.utils.iso8601 import parse_iso8601
-from calibre.utils.date import now as nowf, utcnow, local_tz, isoformat, fromordinal, UNDEFINED_DATE
+from calibre.utils.date import now as nowf, utcnow, local_tz, isoformat, EPOCH, UNDEFINED_DATE
 from calibre.utils.recycle_bin import delete_file
 
 NS = 'http://calibre-ebook.com/recipe_collection'
 E = ElementMaker(namespace=NS, nsmap={None:NS})
 
+
 def iterate_over_builtin_recipe_files():
     exclude = ['craigslist', 'toronto_sun']
     d = os.path.dirname
@@ -58,6 +59,7 @@ def serialize_recipe(urn, recipe_class):
         'description'        : attr('description', '')
         })
 
+
 def serialize_collection(mapping_of_recipe_classes):
     collection = E.recipe_collection()
     '''for u, x in mapping_of_recipe_classes.items():
@@ -80,6 +82,7 @@ def serialize_collection(mapping_of_recipe_classes):
     return etree.tostring(collection, encoding='utf-8', xml_declaration=True,
             pretty_print=True)
 
+
 def serialize_builtin_recipes():
     from calibre.web.feeds.recipes import compile_recipe
     recipe_mapping = {}
@@ -95,9 +98,11 @@ def serialize_builtin_recipes():
 
     return serialize_collection(recipe_mapping)
 
+
 def get_builtin_recipe_collection():
     return etree.parse(P('builtin_recipes.xml', allow_user_override=False)).getroot()
 
+
 def get_custom_recipe_collection(*args):
     from calibre.web.feeds.recipes import compile_recipe, \
             custom_recipes
@@ -122,6 +127,7 @@ def get_custom_recipe_collection(*args):
 def update_custom_recipe(id_, title, script):
     update_custom_recipes([(id_, title, script)])
 
+
 def update_custom_recipes(script_ids):
     from calibre.web.feeds.recipes import custom_recipes, \
             custom_recipe_filename
@@ -151,6 +157,7 @@ def update_custom_recipes(script_ids):
 def add_custom_recipe(title, script):
     add_custom_recipes({title:script})
 
+
 def add_custom_recipes(script_map):
     from calibre.web.feeds.recipes import custom_recipes, \
             custom_recipe_filename
@@ -190,6 +197,7 @@ def remove_custom_recipe(id_):
         except:
             pass
 
+
 def get_custom_recipe(id_):
     from calibre.web.feeds.recipes import custom_recipes
     id_ = str(int(id_))
@@ -200,9 +208,11 @@ def get_custom_recipe(id_):
         with open(os.path.join(bdir, fname), 'rb') as f:
             return f.read().decode('utf-8')
 
+
 def get_builtin_recipe_titles():
     return [r.get('title') for r in get_builtin_recipe_collection()]
 
+
 def download_builtin_recipe(urn):
     from calibre.utils.config_base import prefs
     from calibre.utils.https import get_https_resource_securely
@@ -210,10 +220,12 @@ def download_builtin_recipe(urn):
     return bz2.decompress(get_https_resource_securely(
         'https://code.calibre-ebook.com/recipe-compressed/'+urn, headers={'CALIBRE-INSTALL-UUID':prefs['installation_uuid']}))
 
+
 def get_builtin_recipe(urn):
     with zipfile.ZipFile(P('builtin_recipes.zip', allow_user_override=False), 'r') as zf:
         return zf.read(urn+'.recipe')
 
+
 def get_builtin_recipe_by_title(title, log=None, download_recipe=False):
     for x in get_builtin_recipe_collection():
         if x.get('title') == title:
@@ -232,6 +244,7 @@ def get_builtin_recipe_by_title(title, log=None, download_recipe=False):
                         'Failed to download recipe, using builtin version')
             return get_builtin_recipe(urn)
 
+
 def get_builtin_recipe_by_id(id_, log=None, download_recipe=False):
     for x in get_builtin_recipe_collection():
         if x.get('id') == id_:
@@ -250,6 +263,7 @@ def get_builtin_recipe_by_id(id_, log=None, download_recipe=False):
                         'Failed to download recipe, using builtin version')
             return get_builtin_recipe(urn)
 
+
 class SchedulerConfig(object):
 
     def __init__(self):
@@ -298,7 +312,7 @@ class SchedulerConfig(object):
                     self.root.remove(x)
                     break
             if last_downloaded is None:
-                last_downloaded = fromordinal(1)
+                last_downloaded = EPOCH
             sr = E.scheduled_recipe({
                 'id' : recipe.get('id'),
                 'title': recipe.get('title'),
@@ -424,14 +438,20 @@ class SchedulerConfig(object):
             return utcnow() - ld > timedelta(sch)
         elif typ == 'day/time':
             now = nowf()
-            ld_local = ld.astimezone(local_tz)
+            try:
+                ld_local = ld.astimezone(local_tz)
+            except Exception:
+                return False
             day, hour, minute = sch
             return is_weekday(day, now) and \
                     not was_downloaded_already_today(ld_local, now) and \
                     is_time(now, hour, minute)
         elif typ == 'days_of_week':
             now = nowf()
-            ld_local = ld.astimezone(local_tz)
+            try:
+                ld_local = ld.astimezone(local_tz)
+            except Exception:
+                return False
             days, hour, minute = sch
             have_day = False
             for day in days:
@@ -443,7 +463,10 @@ class SchedulerConfig(object):
                     is_time(now, hour, minute)
         elif typ == 'days_of_month':
             now = nowf()
-            ld_local = ld.astimezone(local_tz)
+            try:
+                ld_local = ld.astimezone(local_tz)
+            except Exception:
+                return False
             days, hour, minute = sch
             have_day = now.day in days
             return have_day and \
